from masint.util.make_api_url import make_api_url

import matplotlib.pyplot as plt

import os
import aiohttp
import asyncio

import traceback

import logging

logger = logging.getLogger(__name__)


def plot(models, smooth, y_limit=None):

    if len(models) == 0:
        models = ["latest"]

    logger.info(f"Plotting models {models}")

    try:
        asyncio.run(plot_async(models=models, smooth=smooth, y_limit=y_limit))
    except Exception as e:
        logger.error(f"Failed to plot model {model_name}")
        logger.error(e)
        logger.error(traceback.format_exc())


async def plot_async(models, smooth, y_limit):
    histories = []
    model_names = []

    for model_name in models:

        status = await get_status(model_name)

        history = status["job_status"]["history"]
        model_name = clip_model_name(
            os.path.basename(status["job_config"]["job_directory"])
        )

        histories.append(history)
        model_names.append(model_name)

    # Plot loss against step
    plot_loss(histories, model_names, smooth, y_limit)


def plot_loss(histories, model_names, smooth, y_limit):

    for history, model_name in zip(histories, model_names):
        steps = [int(entry["step"]) for entry in history]
        losses = [float(entry["loss"]) for entry in history]
        losses = apply_smoothing(losses, smooth)
        plt.plot(steps, losses, label=model_name)

    model_name = model_names[0]

    plt.plot(steps, losses)
    plt.xlabel("Step")
    plt.ylabel("Loss")
    plt.title("Training loss for model " + model_name)
    plt.grid(True)
    if y_limit is not None:
        plt.ylim(0, y_limit)

    base_path = "/app/cray/data"

    if os.path.isdir(base_path):
        path = f"{base_path}/loss_plot_{model_name}.pdf"
    else:
        path = f"loss_plot_{model_name}.pdf"

    logger.info(f"Saving plot to {path}")

    plt.savefig(path)


def apply_smoothing(data, smooth):
    # Number of steps to smooth over
    smooth_steps = smooth

    smoothed_data = []

    for i in range(len(data)):
        if i < smooth_steps:
            smoothed_data.append(sum(data[: i + 1]) / (i + 1))
        else:
            smoothed_data.append(sum(data[i - smooth_steps : i]) / smooth_steps)

    return smoothed_data


async def get_status(model_name):
    async with aiohttp.ClientSession() as session:
        async with session.get(make_api_url(f"v1/megatron/train/{model_name}")) as resp:
            data = await resp.json()

            logger.info(f"Got status for model {model_name}")
            logger.info(data)

            if resp.status != 200:
                logger.error(f"Failed to get status for model {model_name}")
                logger.error(data)
                raise Exception("Failed to get status for model")

            return data


def clip_model_name(model_name):
    return model_name[0:10]



from masint.util.make_api_url import make_api_url
from masint.engines.cray.submit_training_job import upload_async, tar_info_strip_file_info, make_data_file

import tempfile
import tarfile
import contextlib
import os

import logging

logger = logging.getLogger(__name__)

async def submit_slurm_job(code, train_args, api_url):
    with make_slurm_archive(code) as archive_path:
        api_url = make_api_url("v1/megatron/train", api_url=api_url)

        return await upload_async(archive_path, api_url, train_args)

@contextlib.contextmanager
def make_slurm_archive(code):

    data = [{"input": "example input", "output": "example output"}]

    with make_data_file(data) as data_file_path:
        with tempfile.NamedTemporaryFile() as archive_file:
            with tarfile.open(archive_file.name, "w") as tar:
                tar.add(
                    data_file_path,
                    arcname="dataset.jsonlines",
                    filter=tar_info_strip_file_info,
                )

                with make_ml_dir(code) as ml_dir:
                    tar.add(ml_dir, arcname="ml", filter=tar_info_strip_file_info)

            archive_file.seek(0)
            archive_file.flush()

            logger.debug(f"Archive created at {archive_file.name}")
            logger.debug(f"Archive size: {os.path.getsize(archive_file.name)} bytes")

            yield archive_file.name

@contextlib.contextmanager
def make_ml_dir(code):
    """
    Create a temporary directory for the machine learning code.
    """
    with tempfile.TemporaryDirectory() as temp_dir:
        ml_dir = os.path.join(temp_dir, "ml", "cray_megatron")
        os.makedirs(ml_dir, exist_ok=True)

        # Write the code to a file in the ml directory
        code_file_path = os.path.join(ml_dir, "main.py")
        with open(code_file_path, "w") as code_file:
            code_file.write(code)

        yield os.path.dirname(ml_dir)



from masint.util.make_api_url import make_api_url

import masint

import aiohttp
import asyncio
import json
import jsonlines
import io

import logging

logger = logging.getLogger(__name__)


def view_logs(model_name: str, tail: bool, lines: int, follow: bool):
    logger.debug(f"Viewing logs for model {model_name}")

    asyncio.run(
        view_logs_async(model_name=model_name, tail=tail, lines=lines, follow=follow)
    )


async def view_logs_async(model_name: str, tail: bool, lines: int, follow: bool):
    already_printed_lines = set()

    starting_line_number = 0

    while True:
        logs = await sample_log_stream(
            model_name=model_name, starting_line_number=starting_line_number
        )

        if tail:
            logs = logs[-lines:]

        for log in logs:
            if log["line_number"] in already_printed_lines:
                continue

            print(format_log(log))
            already_printed_lines.add(log["line_number"])

            if log["line_number"] > starting_line_number:
                starting_line_number = log["line_number"]

        if not follow:
            break

        await asyncio.sleep(1)


async def sample_log_stream(model_name: str, starting_line_number: int):
    async with aiohttp.ClientSession() as session:
        async with session.get(
            make_api_url(f"v1/megatron/train/logs/{model_name}"),
            params={"starting_line_number": starting_line_number},
        ) as resp:
            if resp.content_type != "text/event-stream":
                # The response is json
                response = await resp.json()

                raise Exception(f"Error {resp.status} getting log stream: {response}")

            object_buffer = await read_log_stream(resp.content)

    return object_buffer


async def read_log_stream(log_stream):
    object_buffer = []

    async for chunk in log_stream.iter_any():
        text = chunk.decode("utf-8")

        try:
            reader = jsonlines.Reader(io.StringIO(text))

            for obj in reader:
                object_buffer.append(obj)
        except Exception:
            logger.debug(f"Failed to decode json: {text}")
            continue

    return object_buffer


def format_log(log):
    return f"{log['line_number']} {log['line'].rstrip()}"



from masint.util.make_api_url import make_api_url

import aiofiles
import aiohttp
import contextlib
import asyncio
import io
import json
import os
import tarfile
import tempfile
import jsonlines

import logging

logger = logging.getLogger(__name__)


async def submit_training_job(data, model_name, train_args, api_url):
    with make_training_archive(data) as archive_path:
        api_url = make_api_url("v1/megatron/train", api_url=api_url)

        return await upload_async(archive_path, api_url, train_args)


async def upload_async(data_file_path, api_url, train_args):
    async with aiohttp.ClientSession() as session:

        content_length = await get_content_length(data_file_path, train_args)

        with make_multipart_writer(data_file_path, train_args) as mp:

            headers = mp.headers

            headers["Content-Length"] = str(content_length)

            async with session.post(api_url, data=mp, headers=headers) as resp:
                if resp.status != 200:
                    raise Exception(f"Failed to upload data: {await resp.text()}")
                return await resp.json()


async def get_content_length(data_file_path, train_args):
    with make_multipart_writer(data_file_path, train_args) as mp:

        class Writer:
            def __init__(self):
                self.count = 0

            async def write(self, data):
                self.count += len(data)

        writer = Writer()
        await mp.write(writer)
        content_length = writer.count

        return content_length


@contextlib.contextmanager
def make_multipart_writer(data_file_path, train_args):
    with aiohttp.MultipartWriter("form-data") as mp:
        file_part = mp.append(file_sender(data_file_path))
        file_part.set_content_disposition("form-data", name="file", filename="dataset")

        params_part = mp.append_json(train_args)
        params_part.set_content_disposition("form-data", name="params")

        yield mp


async def file_sender(file_path):
    chunk_size = 64 * 1024  # 64 KB

    async with aiofiles.open(file_path, "rb") as f:
        chunk = await f.read(chunk_size)
        index = 0
        while chunk:
            yield chunk
            chunk = await f.read(chunk_size)
            index += chunk_size


@contextlib.contextmanager
def make_training_archive(data):

    with make_data_file(data) as data_file_path:
        check_for_zero_length_file(data_file_path)

        with tempfile.NamedTemporaryFile() as archive_file:
            with tarfile.open(archive_file.name, "w") as tar:
                # Add the data file to the archive
                tar.add(
                    data_file_path,
                    arcname="dataset.jsonlines",
                    filter=tar_info_strip_file_info,
                )

                # Add the machine learning directory to the archive
                ml_dir = find_ml_dir()

                if ml_dir is None:
                    logger.warning(
                        f"ML directory not found. Skipping addition to "
                        "archive, using default ml directory from ScalarLM server."
                    )
                else:
                    tar.add(ml_dir, arcname="ml", filter=tar_info_strip_file_info)

            archive_file.seek(0)
            archive_file.flush()

            logger.debug(f"Archive created at {archive_file.name}")
            logger.debug(f"Archive size: {os.path.getsize(archive_file.name)} bytes")

            yield archive_file.name


def tar_info_strip_file_info(tarinfo):
    tarinfo.uid = tarinfo.gid = 0
    tarinfo.uname = tarinfo.gname = "root"
    tarinfo.mtime = 0
    return tarinfo


def find_ml_dir():
    # Check the current directory first
    current_directory = os.path.join(os.getcwd(), "ml")

    if os.path.exists(current_directory):
        return current_directory

    # The directory tree is as follows:
    #  - cray/sdk/masint/engines/cray/submit_training_job.py <- this file
    #  - cray/ml <- the machine learning directory
    peer_directory = os.path.join(
        os.path.dirname(__file__), "..", "..", "..", "..", "ml"
    )

    if os.path.exists(peer_directory):
        return peer_directory


@contextlib.contextmanager
def make_data_file(data):

    if isinstance(data, str):
        with open(data, "rb") as f:
            yield f

    elif isinstance(data, io.BufferedIOBase):
        chunk_size = 64 * 1024

        with tempfile.NamedTemporaryFile() as f:
            for chunk in iter(lambda: data.read(chunk_size), b""):
                f.write(chunk)
            f.seek(0)
            f.flush()
            yield f.name

    elif isinstance(data, list):
        with tempfile.NamedTemporaryFile() as f:
            with jsonlines.open(f.name, mode="w") as writer:
                for item in data:
                    writer.write(item)

            yield f.name

    else:
        raise ValueError(f"Unsupported data type: {type(data)}")

def check_for_zero_length_file(file_path):
    if os.path.getsize(file_path) == 0:
        raise ValueError(f"The file {file_path} is empty. Please provide valid data.")



from masint import SupermassiveIntelligence

import logging

logger = logging.getLogger(__name__)


def stats():
    logger.debug(f"Getting stats")

    try:
        llm = SupermassiveIntelligence()

        stats = llm.metrics()
    except Exception as e:
        logger.error(f"Failed to get stats")
        logger.error(e)
        return

    print_stats(stats)


def print_stats(stats):
    keys = stats.keys()

    max_lengths = {key: len(key) for key in keys}

    for key in keys:
        if "flop" in key:
            stats[key] = humanize_flops(stats[key])
        elif "/s" in key or "_time" in key:
            stats[key] = f"{stats[key]:.2f}"

    for key, value in stats.items():
        max_lengths[key] = max(max_lengths[key], len(str(value)))

    header = " | ".join(f"{key:<{max_lengths[key]}}" for key in keys)

    print(header)
    print("-" * len(header))
    row = " | ".join(f"{str(stats[key]):<{max_lengths[key]}}" for key in keys)
    print(row)

def humanize_flops(flops):
    """
    Convert FLOPS to a human-readable format.
    """
    if flops < 1e9:
        return f"{flops:.2f} FLOPS"
    elif flops < 1e12:
        return f"{flops / 1e9:.2f} GFLOPS"
    elif flops < 1e15:
        return f"{flops / 1e12:.2f} TFLOPS"
    elif flops < 1e18:
        return f"{flops / 1e15:.2f} PFLOPS"
    else:
        return f"{flops / 1e18:.2f} EFLOPS"



from masint.util.get_api_base import get_api_base


def make_api_url(endpoint, api_url=None):
    if api_url is not None:
        api_base = api_url
    else:
        api_base = get_api_base()
    return f"{api_base}/{endpoint}"



from masint.util.make_api_url import make_api_url

import aiohttp
import json
import tempfile
import logging

logger = logging.getLogger(__name__)


async def poll_for_downloads(result, api_url=None):
    api_url = make_api_url("v1/generate/download", api_url=api_url)

    request_id = result["request_id"]

    async with aiohttp.ClientSession() as session:
        final_result = None

        while not is_download_finished(final_result):
            async with session.post(api_url, json={"request_id": request_id}) as response:
                total_size = int(response.headers.get("content-length", 0))
                downloaded = 0

                with tempfile.NamedTemporaryFile(mode="w+", delete=True) as f:
                    logger.debug(f"Created temporary download file at {f.name}")

                    async for chunk in response.content.iter_chunked(64 * 1024):
                        if chunk:
                            f.write(chunk.decode("utf-8"))
                            downloaded += len(chunk)
                            logger.debug(
                                f"Downloaded {downloaded} of {total_size} bytes"
                                " ({(downloaded/total_size)*100:.2f}%)"
                            )

                    f.flush()
                    f.seek(0)

                    final_result = json.load(f)

    # convert the results from a dict to a list
    if "results" in final_result:
        results_list = []
        for key, response in final_result["results"].items():
            results_list.append(response)
        final_result["results"] = results_list

    return final_result


def is_download_finished(result):
    if result is None:
        return False

    if not "current_index" in result:
        logger.warn(f"No current_index in download result : {result}")
        return False

    if not "total_requests" in result:
        logger.warn(f"No total_requests in download result : {result}")
        return False

    if result["current_index"] < result["total_requests"]:
        return False

    if not "results" in result:
        logger.warn(f"No results in download result : {result}")
        return False

    for key, response in result["results"].items():
        if "error" in response and response["error"] is not None:
            raise Exception(response["error"])

    return True



from masint.api.supermassive_intelligence import SupermassiveIntelligence

import traceback

import logging

logger = logging.getLogger(__name__)


def cancel(model_name):

    logger.info(f"Cancelling model: {model_name}")

    try:
        smi = SupermassiveIntelligence()
        response = smi.cancel(model_name)
        logger.info(f"Cancel response: {response}")
    except Exception as e:
        logger.error(f"Failed to cancel model: {model_name}")
        logger.error(e)
        logger.error(traceback.format_exc())




from masint.api.supermassive_intelligence import SupermassiveIntelligence

import traceback

import logging

logger = logging.getLogger(__name__)


def clear_queue():

    logger.info(f"Clearing inference queue...")

    try:
        smi = SupermassiveIntelligence()
        response = smi.clear_queue()
        logger.info(f"Inference queue cleared: {response}")
    except Exception as e:
        logger.error(f"Failed to clear inference queue.")
        logger.error(e)
        logger.error(traceback.format_exc())



from masint.util.make_api_url import make_api_url

from masint.engines.cray.submit_training_job import submit_training_job
from masint.engines.cray.submit_slurm_job import submit_slurm_job

from masint.engines.cray.upload_generate import upload_generate
from masint.engines.cray.poll_for_downloads import poll_for_downloads

import aiohttp

import logging

logger = logging.getLogger(__name__)


class AsyncCray:
    def __init__(self, api_url=None):
        self.api_url = api_url

    async def train(self, data, model_name, train_args):
        return await submit_training_job(
            data, model_name, train_args, api_url=self.api_url
        )

    async def submit_slurm_job(self, code, train_args=None):
        return await submit_slurm_job(code, train_args, api_url=self.api_url)

    async def generate(self, prompts, model_name, max_tokens):

        upload_threshold = 128

        if len(prompts) > upload_threshold:
            result = await upload_generate(
                prompts, model_name, max_tokens, api_url=self.api_url
            )

            handle_upload_error(result)

            final_result = await poll_for_downloads(result, api_url=self.api_url)
        else:
            result = await self.submit_generate(prompts, model_name, max_tokens)

            handle_error(result)

            final_result = await poll_for_responses(result, api_url=self.api_url)

        return [response["response"] for response in final_result["results"]]

    async def submit_generate(self, prompts, model_name, max_tokens):
        api_url = make_api_url("v1/generate", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            params = {"prompts": prompts}

            if model_name is not None:
                params["model"] = model_name

            if max_tokens is not None:
                params["max_tokens"] = max_tokens

            async with session.post(api_url, json=params) as resp:
                assert resp.status == 200
                return await resp.json()

    async def get_results(self, request_ids):
        async with aiohttp.ClientSession() as session:
            api_url = make_api_url("v1/generate/get_results", api_url=self.api_url)
            async with session.post(api_url, json={"request_ids": request_ids}) as resp:
                assert resp.status == 200
                return await resp.json()

    async def list_models(self):
        api_url = make_api_url("v1/megatron/list_models", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                return await resp.json()

    async def get_training_job(self, job_dir):
        api_url = make_api_url(f"v1/megatron/train/{job_dir}", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                return await resp.json()

    async def health(self):
        api_url = make_api_url("v1/health", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                return await resp.json()

    async def metrics(self):
        api_url = make_api_url("v1/generate/metrics", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                return await resp.json()

    async def get_gpu_count(self):
        api_url = make_api_url("v1/megatron/gpu_count", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                response = await resp.json()
                logger.debug(f"get_gpu_count response: {response}")
                return response["gpu_count"]

    async def get_node_count(self):
        api_url = make_api_url("v1/megatron/node_count", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as resp:
                response = await resp.json()
                logger.debug(f"get_node_count response: {response}")
                return response["node_count"]

    async def cancel(self, model_name):
        api_url = make_api_url(f"v1/megatron/cancel/{model_name}", api_url=self.api_url)
        async with aiohttp.ClientSession() as session:
            async with session.post(api_url) as resp:
                return await resp.json()

    async def clear_queue(self):
        api_url = make_api_url("v1/generate/clear_queue", api_url=self.api_url)

        async with aiohttp.ClientSession() as session:
            async with session.post(api_url) as resp:
                return await resp.json()


def handle_error(result):

    if "error" in result and result["error"] is not None:
        logger.error(f"Error in response: {result['error']}")
        raise Exception(result["error"])

    if not result.get("results"):
        logger.error(f"No results found in response: {result}")
        raise Exception("No results found in response")

    if not isinstance(result["results"], list):
        logger.error(f"Results is not a list: {result['results']}")
        raise Exception("Results is not a list")


def handle_upload_error(result):
    if "error" in result and result["error"] is not None:
        logger.error(f"Error in response: {result['error']}")
        raise Exception(result["error"])


async def poll_for_responses(result, api_url):
    api_url = make_api_url("v1/generate/get_results", api_url=api_url)

    async with aiohttp.ClientSession() as session:
        while not is_finished(result):
            request_ids = [response["request_id"] for response in result["results"]]
            async with session.post(api_url, json={"request_ids": request_ids}) as resp:
                assert resp.status == 200
                result = await resp.json()

            handle_error(result)

    return result


def is_finished(result):
    for response in result["results"]:
        if response["error"] is not None:
            raise Exception(response["error"])

        if response["response"] is None:
            return False

    return True



from masint import SupermassiveIntelligence

from datetime import datetime, timedelta

from humanize import naturaldelta

import logging

logger = logging.getLogger(__name__)


def ls(all, limit=None):
    logger.debug(f"Listing models")

    try:
        llm = SupermassiveIntelligence()

        models = llm.list_models()
    except Exception as e:
        logger.error(f"Failed to list models")
        logger.error(e)

    if limit is not None:
        models["models"] = models["models"][:limit]

    if all:
        print_all_model_info(models["models"])
    else:
        print_models(models["models"])


def print_models(models):
    for model in models:
        print(model["name"])


def print_all_model_info(models):
    if len(models) == 0:
        return

    keys = models[0].keys()

    max_lengths = {key: len(key) for key in keys}

    for model in models:
        for key in keys:
            if  'train_time' in key:
                model[key] = naturaldelta(timedelta(seconds=model[key]))
            elif 'time' in key:
                model[key] = datetime.fromtimestamp(model[key]).strftime('%Y-%m-%d %H:%M:%S')

    for model in models:
        for key, value in model.items():
            max_lengths[key] = max(max_lengths[key], len(str(value)))

    header = " | ".join(f"{key:<{max_lengths[key]}}" for key in keys)

    print(header)
    print("-" * len(header))
    for model in models:
        row = " | ".join(f"{str(model[key]):<{max_lengths[key]}}" for key in keys)
        print(row)




from masint.util.make_api_url import make_api_url

import aiohttp
import aiofiles
import contextlib
import tempfile
import json

import logging

logger = logging.getLogger(__name__)


async def upload_generate(prompts, model_name, max_tokens, api_url=None):

    with make_upload_json_file(prompts, model_name, max_tokens) as upload_path:

        api_url = make_api_url("v1/generate/upload", api_url=api_url)

        return await upload_async(upload_path, api_url)


@contextlib.contextmanager
def make_upload_json_file(prompts, model_name, max_tokens):
    requests_object = [
        {
            "prompt": prompt,
            "model": model_name,
            "max_tokens": max_tokens,
            "request_type": "generate",
        }
        for prompt in prompts
    ]

    with tempfile.NamedTemporaryFile(mode="w+", delete=True) as f:
        json.dump(requests_object, f)
        f.flush()
        f.seek(0)

        logger.debug(f"Created temporary upload file at {f.name}")
        logger.debug(f"Upload file size: {f.tell()} bytes")

        yield f.name


async def upload_async(data_file_path, api_url):
    async with aiohttp.ClientSession() as session:

        content_length = await get_content_length(data_file_path)

        with make_multipart_writer(data_file_path) as mp:

            headers = mp.headers

            headers["Content-Length"] = str(content_length)

            async with session.post(api_url, data=mp, headers=headers) as resp:
                if resp.status != 200:
                    raise Exception(
                        f"Failed to upload generate data: {await resp.text()}"
                    )
                return await resp.json()


async def get_content_length(data_file_path):
    with make_multipart_writer(data_file_path) as mp:

        class Writer:
            def __init__(self):
                self.count = 0

            async def write(self, data):
                self.count += len(data)

        writer = Writer()
        await mp.write(writer)
        content_length = writer.count

        return content_length


@contextlib.contextmanager
def make_multipart_writer(data_file_path):
    with aiohttp.MultipartWriter("form-data") as mp:
        part = mp.append(file_sender(data_file_path))
        part.set_content_disposition("form-data", name="file", filename="requests.json")

    yield mp


async def file_sender(file_path):
    chunk_size = 64 * 1024

    async with aiofiles.open(file_path, "rb") as f:
        chunk = await f.read(chunk_size)
        index = 0
        while chunk:
            yield chunk
            chunk = await f.read(chunk_size)
            index += chunk_size



import masint
import scalarlm

import os

def get_api_base():
    if hasattr(scalarlm, "api_url") and scalarlm.api_url is not None:
        return scalarlm.api_url

    if hasattr(masint, "api_url") and masint.api_url is not None:
        return masint.api_url

    if "SCALARLM_API_URL" in os.environ:
        return os.environ["SCALARLM_API_URL"]

    if "MASINT_API_URL" in os.environ:
        return os.environ["MASINT_API_URL"]

    return "http://localhost:8000"




from masint.cli.view_logs import view_logs
from masint.cli.view_service_logs import view_service_logs
from masint.cli.plot import plot
from masint.cli.ls import ls
from masint.cli.squeue import squeue
from masint.cli.stats import stats
from masint.cli.clear_queue import clear_queue
from masint.cli.cancel import cancel

import masint

from argparse import ArgumentParser

import logging

logger = logging.getLogger(__name__)


def main():
    setup_logging()

    arguments, parser = parse_arguments()

    if arguments.command == "logs":
        if arguments.service_name:
            view_service_logs(
                service_name=arguments.service_name,
                tail=arguments.tail,
                follow=arguments.follow,
                lines=arguments.lines,
            )
        else:
            # Default behavior: training job logs
            view_logs(
                model_name=arguments.model,
                tail=arguments.tail,
                lines=arguments.lines,
                follow=arguments.follow,
            )

    elif arguments.command == "plot":
        plot(models=arguments.model, smooth=int(arguments.smooth), y_limit=arguments.y_limit)

    elif arguments.command == "ls":
        ls(all=arguments.all, limit=arguments.limit)

    elif arguments.command == "squeue":
        squeue()

    elif arguments.command == "stats":
        stats()

    elif arguments.command == "clear_queue":
        clear_queue()

    elif arguments.command == "cancel":
        cancel(model_name=arguments.model)

    else:
        logger.error(f"Unknown command {arguments.command}")
        parser.print_help()
        exit(1)


def setup_logging():
    logging.basicConfig(level=logging.WARNING)

    logging.getLogger("matplotlib").setLevel(logging.WARNING)
    logging.getLogger("asyncio").setLevel(logging.WARNING)


def parse_arguments():
    parser = ArgumentParser(description="The command line interface for MasInt ScalarLM")

    # Add a subparser for the 'logs' command
    subparsers = parser.add_subparsers(dest="command")

    add_logs_parser(subparsers)
    add_plot_parser(subparsers)
    add_ls_parser(subparsers)
    add_squeue_parser(subparsers)
    add_stats_parser(subparsers)
    add_clear_queue_parser(subparsers)
    add_cancel_parser(subparsers)

    args = parser.parse_args()

    return args, parser


def add_logs_parser(subparsers):
    logs_parser = subparsers.add_parser("logs", help="View logs")

    # Add optional positional argument for service name
    logs_parser.add_argument(
        "service_name",
        nargs="?",  # Makes it optional
        default=None,
        help="The name of the service to view logs for (optional)"
    )

    logs_parser.add_argument("--model", help="The model to view logs for", default="latest")
    logs_parser.add_argument(
        "--tail",
        help="Whether to tail the logs",
        default=False,
        action="store_true",
    )
    logs_parser.add_argument(
        "--follow",
        "-f",
        help="Whether to follow the logs",
        default=False,
        action="store_true",
    )

    logs_parser.add_argument(
        "--lines", type=int, help="The number of lines to view", default=100
    )


def add_plot_parser(subparsers):
    plot_parser = subparsers.add_parser("plot", help="Plot the results of a model")

    plot_parser.add_argument("--model", help="The model to plot results for", default=[], action="append")
    plot_parser.add_argument("--smooth", help="The number of steps to smooth over", default=1)
    plot_parser.add_argument("--y-limit", help="The y-axis limit for the plot", default=None, type=float)


def add_ls_parser(subparsers):
    ls_parser = subparsers.add_parser("ls", help="List models")
    ls_parser.add_argument("-A", "--all", help="List all attributes of the models", default=False, action="store_true")
    ls_parser.add_argument("-l", "--limit", help="Limit the number of models returned", default=None, type=int)


def add_squeue_parser(subparsers):
    squeue_parser = subparsers.add_parser("squeue", help="View the squeue")

def add_stats_parser(subparsers):
    stats_parser = subparsers.add_parser("stats", help="View the stats of the models")

def add_clear_queue_parser(subparsers):
    clear_queue_parser = subparsers.add_parser("clear_queue", help="Clear the inference queue")

def add_cancel_parser(subparsers):
    cancel_parser = subparsers.add_parser("cancel", help="Cancel a running model")
    cancel_parser.add_argument("--model", help="The model to cancel", required=True)


if __name__ == "__main__":
    main()
